#ifndef __DEFAULT_CUBEMAP_SCENE_WIDGET_H__
#define __DEFAULT_CUBEMAP_SCENE_WIDGET_H__


// Includes Projection Designer headers
#include <projdesigner/include/interfaces.h>

// Includes the ui_ header
#include "ui_defaultcubemapscenewidget.h"

// Includes local headers
#include "DefaultCubemapSceneContent.h"


/**
 * Implementation of the DefaultCubemapSceneWidget class.
 *
 * This widget is used in a stacked widget with all the other scenes,
 * to configure the CubeMap.
 *
 */
class DefaultCubemapSceneWidget : public SceneWidget, private Ui::DefaultCubemapSceneWidget
{
    Q_OBJECT
		

public:


    /**
     * Default constructor.
     *
     */
    DefaultCubemapSceneWidget(SceneInterface *plugin, QWidget* pParent=0, Qt::WFlags flag=0);


    /**
     * Destructor.
     *
     */
    ~DefaultCubemapSceneWidget();


    /**
     * Updates the widget.
     *
     * This method can be used it two ways: to force a refresh or to update the
     * data of the widget.
     *
     * @param content A const pointer on the contents to be used for the widget.
     *                This contents will be duplicated in the widget. If content
     *                is NULL, the current content are used and the refresh is
     *                forced. If no content are given, NULL is used.
     *
     */
    void updateGUI(const SceneContent* content=NULL);


    /**
     * Gets the widget contents.
     *
     * @return A const reference on the content of the widget.
     *
     */
    const SceneContent& content(void) const {return m_content;}


signals:


    /**
     * [signal] Notify that the contents of the widget have changed.
     *
     * This signal is emitted each time that a field is changed.
     *
     * @param content A const reference on the new contents (the same as 
     *                returned by the content() method).
     *
     */
    virtual void dataChanged(const SceneContent& content);


private slots:


    /**
     * [private slot] Called when the "front file" field is edited.
     *
     */
    void on_frontFileNameEdit_editingFinished(void);


    /**
     * [private slot] Called when the "front file" button is pressed.
     *
     */
    void on_frontBrowseButton_clicked(void);


    /**
     * [private slot] Called when the "back file" field is edited.
     *
     */
    void on_backFileNameEdit_editingFinished(void);


    /**
     * [private slot] Called when the "back file" button is pressed.
     *
     */
    void on_backBrowseButton_clicked(void);


    /**
     * [private slot] Called when the "left file" field is edited.
     *
     */
    void on_leftFileNameEdit_editingFinished(void);


    /**
     * [private slot] Called when the "left file" button is pressed.
     *
     */
    void on_leftBrowseButton_clicked(void);


    /**
     * [private slot] Called when the "right file" field is edited.
     *
     */
    void on_rightFileNameEdit_editingFinished(void);


    /**
     * [private slot] Called when the "right file" button is pressed.
     *
     */
    void on_rightBrowseButton_clicked(void);


    /**
     * [private slot] Called when the "top file" field is edited.
     *
     */
    void on_topFileNameEdit_editingFinished(void);


    /**
     * [private slot] Called when the "top file" button is pressed.
     *
     */
    void on_topBrowseButton_clicked(void);


    /**
     * [private slot] Called when the "bottom file" field is edited.
     *
     */
    void on_bottomFileNameEdit_editingFinished(void);


    /**
     * [private slot] Called when the "bottom file" button is pressed.
     *
     */
    void on_bottomBrowseButton_clicked(void);


private:


    /// The current DefaultCubemapSceneContent of the widget.
    DefaultCubemapSceneContent m_content;


};

#endif // __DEFAULT_CUBEMAP_SCENE_WIDGET_H__
